var Index = function()
{
	var index = {};
	var debug = 0;
	var myPermissions = {};
	var counterInfo = null;

	index.initPage = function()
	{
		Events.doOnEnter('caseName', index.caseSummary);
		Events.addEventHandler('caseName', 'keypress', index.testSnake);
		Buttons.addClickHandler('btnCaseSummary', index.caseSummary);
		index.initPermissions();
		index.loadCounts();
		
		var frm = document.forms['reggie'];
		frm.caseName.disabled = !App.getActiveProjectId();
	}
	
	index.initPermissions = function()
	{
		// Get permission information
		var url = 'Install.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPermissions';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, index.onPermissionsLoaded, index.onPermissionsLoaded);
		request.send(null);
	}
	
	// Callback when permission information is ready
	index.onPermissionsLoaded = function(request)
	{
		if (debug) App.debug(Strings.encodeTags(request.responseText));
		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		
		if (error)
		{
			alert(error);
			return;
		}
		
		myPermissions = response.permissions;
		var experimentalFeatures = response.experimentalFeatures;
		var isAdmin = myPermissions['Administrator'] == 1;
		var noProjectIsActive = !App.getActiveProjectId();
		
		// Enable links based on the permission
		var wizards = document.getElementsByClassName('require-permission');
		// NOTE! Loop backwards since changing the classname 
		// of an element so that 'require-permission' no longer is included
		// removes the element from the 'wizards' list
		for (var wizardNo = wizards.length-1; wizardNo >= 0; wizardNo--)
		{
			var wizard = wizards[wizardNo];
			if (wizard.className.indexOf('experimental') >= 0)
			{
				if (experimentalFeatures[wizard.id] == 1) Doc.removeClass(wizard, 'not-implemented');
			}
			
			if (wizard.className.indexOf('not-implemented') == -1)
			{
				var role = wizard.getAttribute('data-role');
				if (noProjectIsActive)
				{
					wizard.title = 'You must select a project before using this wizard';
				}
				else if (!role || myPermissions[role] == 1 || isAdmin)
				{
					wizard.className = 'link';
					wizard.addEventListener('click', index.wizardOnClick);
				}
				else
				{
					wizard.title = role + ' permission is required to use this wizard';
				}
			}
		}
	}

	index.wizardOnClick = function(event)
	{
		var role = Data.get(event.currentTarget, 'role');
		var isAdmin = myPermissions['Administrator'] == 1;
		if (!role || myPermissions[role] == 1 || isAdmin)
		{
			var link = Data.get(event.currentTarget, 'link');
			location.href = link;
		}
		else
		{
			Forms.showNotification(event.currentTarget, 'You do not have permission to use this wizard!', null, null, 'left');
		}
	}

	index.forceLoadCounts = function()
	{
		index.loadCounts(true);
	}
	
	index.loadCounts = function(forceUpdate)
	{
		var url = 'Session.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCounters'
		if (forceUpdate) url += '&forceUpdate=1';
		var request = Ajax.getXmlHttpRequest();
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, index.onCounted, index.onCounted);
		request.send(null);
	}
	
	index.onCounted = function(request)
	{
		if (debug) 
		{
			App.debug(Strings.encodeTags(request.responseText));
		}

		var response;
		var error = false;
		try
		{
			response = JSON.parse(request.responseText);
			if (response.status != 'ok')
			{
				error = response.message || response.stacktrace || 'Unexpected error';
			}
		}
		catch (ex)
		{
			error = ex;
		}
		if (error) App.debug(error);
		
		var counts = error ? null : response.counts;
		if (counts == null) return;

		counterInfo = response;
		counterInfo.infoTime = new Date().getTime();
		
		var counters = document.getElementsByClassName('counter');
		for (var counterNo = 0; counterNo < counters.length; counterNo++)
		{
			var counter = counters[counterNo];
			var countId = Data.get(counter, 'counter');

			var count = counts[countId];
			Doc.addClass(counter, 'done');
			if (count == null)
			{
				counter.innerHTML = '-';
			}
			else
			{
				counter.innerHTML = count < 1000 ? count : Math.floor(count / 1000) + ',' + index.zeroPad(count % 1000);
			}
		}
		
		if (counts['empty-storage-boxes'] < 2)
		{
			Doc.show('more-boxes-are-needed', 'inline-block');
		}
		
		index.updateCounterAge();
	}
	
	// Pad a numeric value with leading zeros so that a string with at least 3 digits is created
	index.zeroPad = function(val)
	{
		if (val < 10) return '00'+val;
		if (val < 100) return '0'+val;
		return val;
	}
	
	index.updateCounterAge = function()
	{
		var counterAge = '<b>Counters last updated</b> ';
		var time = new Date().getTime();
		var infoAgeInSeconds = Math.floor((time - counterInfo.infoTime) / 1000);
		var ageInSeconds = counterInfo.countAge + infoAgeInSeconds;
		var ageInMinutes = Math.floor(ageInSeconds / 60);
		var formattedDate = Reggie.reformatDate(counterInfo.countTime);
		
		if (ageInSeconds < 60)
		{
			// Counters are less than 1 minute old, display age in seconds
			// Update the counter age every 5 seconds
			counterAge += ageInSeconds == 1 ? '1 second ago' : ageInSeconds + ' seconds ago';
			setTimeout(index.updateCounterAge, 1000 * (5 - ageInSeconds % 5));
		}
		else 
		{
			if (ageInMinutes < 60)
			{
				// Counters are less than 1 hour old, display age in minutes
				counterAge += ageInMinutes == 1 ? '1 minute ago' : ageInMinutes + ' minutes ago';
			}
			else
			{
				// Display age as date and time
				counterAge += formattedDate;
			}
			if (ageInMinutes >= 5)
			{
				// Enabled a manual force update of the counter if they are older than 5 minutes
				counterAge += '<img src="images/refresh-small.png" class="link" id="force-update-counters" title="Force update of counters">';
			}
			
			if (infoAgeInSeconds >= 300)
			{
				// Reload counts from the server if the information is more than 5 minutes old
				setTimeout(index.loadCounts, 1000);
			}
			else
			{
				// Otherwise just update the age that is displayed every 15 seconds
				setTimeout(index.updateCounterAge, 15000);
			}
		}
		
		Doc.element('counter-age').innerHTML = counterAge;
		Doc.element('counter-age').title = formattedDate;
		if (ageInMinutes >= 5)
		{
			Events.addEventHandler('force-update-counters', 'click', index.forceLoadCounts);
		}
	}
	
	index.caseSummary = function()
	{
		var frm = document.forms['reggie'];
		
		var caseName = frm.caseName.value;
		if (!caseName)
		{
			Forms.showNotification('caseName', 'Please enter a 7-digit case id', null, null, 'left');
			return;
		}
		var url = 'reports/case_summary.jsp?ID='+App.getSessionId();
		url += '&caseName='+encodeURIComponent(caseName);
		location.href = url;
	}
	
	index.testSnake = function(event)
	{
		if (event.currentTarget.value == 'Snak' && event.charCode == 101)
		{
			setTimeout(index.showSnake, 500);
		}
	}

	index.showSnake = function()
	{
		var msg = '<img src="images/snake-icon-64.png" style="float: right;">'
		msg += '– There\'s a big snake in the plane, Jock!<br>';
		msg += '– Oh, that\'s just my pet snake, Reggie!<br>';
		msg += '– I hate snakes, Jock! I hate \'em!<br>';
		msg += '– Come on! Show a little backbone, will ya?<br>';
		msg += '<div class="snake-quote-source">Indiana Jones - Raiders of the Lost Ark</div>';
		Forms.showNotification('caseName', msg, 'snake-quote', 'pointer-above');
	}

	
	return index;
}();

Doc.onLoad(Index.initPage);

